#!/usr/bin/python3
#
# Name:         bbb_smartmeters_p1.py
# Purpose:      Client to get MQTT data from Mosquitto
# Author:       weigu.lu
# Date:         7/17
#
#-----------------------------------------------------------------------------

import sys
import os
import shutil
import shlex
from time import gmtime, strftime, localtime, sleep
from datetime import datetime, timedelta, time
import paho.mqtt.client as mqtt
import json
import subprocess

ftime = strftime("%Y_%m_%d", localtime())
rftime = (datetime.today() - timedelta(days=1)).strftime("%Y_%m_%d")
ftime2 = strftime("%d.%m.%y", localtime())
ftime3 = strftime("%d.%m.%y %H:%M:%S", localtime())
ftime4 = strftime("%H:%M", localtime())

wpath = "/var/www/png/"
fpath = "/home/weigu/teensylogger/"
gp_dir = fpath+"gp/"
png_dir = fpath+"png/"
png_a_dir = fpath+"png_archive/"
data_dir = fpath+"data/"
data_a_dir = fpath+"data_archive/"
sm_p1_datafile1 = data_dir+"sm_p1_data.min"
sm_p1_datafile2 = data_dir+"sm_p1_data_"
sm_p1_gnupfile1 = gp_dir+"sm_p1_gp_template.gp"
sm_p1_gnupfile2 = gp_dir+"sm_p1.gp"
mailfile = fpath+'mail.txt'
pngfile = png_a_dir+'sm_p1_'+rftime+'.png'

imp1 = 0.06  #Wh/imp Z2-5
imp2 = 2  #Wh/imp Z1 lansis and gear
clientID = "getsmartmeters_p1"
brokerIP = "192.168.1.69"
brokerPort = 1883
topic1  = "basement/smartmeter1_p1"
topic2  = "basement/smartmeter2_p1"
topic3  = "basement/smartmeter3_p1"
topic4  = "basement/smartmeter4_p1"
topic5  = "basement/smartmeter5_p1"
DEBUG = 0
sm2_id="430"
sm3_id="568"
sm4_id="423"
sm5_id="552"

# Callback that is executed when the client receives a CONNACK response from the server.
def onConnect(client, userdata, flags, rc):
   print("Connected with result code " + str(rc))
   mqttc.subscribe([(topic2, 0),(topic3,0),(topic4,0),(topic5,0)])  # Subscribe to the topics (topic name, QoS)

# Callback that is executed when we disconnect from the broker.
def onDisconnect(client, userdata, message):
   print("Disconnected from the broker.")

# Callback that is executed when subscribing to a topic
def onSubscribe(client, userdata, mid, granted_qos):
   print('Subscribed on topic.')
   
# Callback that is executed when unsubscribing to a topic
def onUnsubscribe(client, userdata, mid, granted_qos):
   print('Unsubscribed on topic.')   

# Callback that is executed when a message is received.
def onMessage(client, userdata, message):
    # Subscribing in on_connect() means that if we lose the connection and
    # reconnect then subscriptions will be renewed.
    #print('M',end='')
    #print('message received')
    global sm2, sm3, sm4, sm5, sa          # smartmeter value (sa = solar all)
    global sm2p, sm3p, sm4p, sm5p, sap     # power
    global sm2e, sm3e, sm4e, sm5e, sae     # energy
    global sm2_mn, sm3_mn, sm4_mn, sm5_mn  # smartmetervalue at midnight
    
    now = datetime.now()
    now_time = now.time()
    if (now_time==time(23,59,59)) and (ioj["id"]==sm2_id):
        sm2_mn=ioj["c1"].rstrip("*kWh")
    if (now_time==time(23,59,59)) and (ioj["id"]==sm3_id):
        sm3_mn=ioj["c1"].rstrip("*kWh")
    if (now_time==time(23,59,59)) and (ioj["id"]==sm4_id):
        sm4_mn=ioj["c1"].rstrip("*kWh")
    if (now_time==time(23,59,59)) and (ioj["id"]==sm5_id):
        sm5_mn=ioj["c1"].rstrip("*kWh")
    ftime = strftime("%Y_%m_%d", localtime())
    ftime3 = strftime("%d.%m.%y %H:%M:%S", localtime())
    io=message.payload.decode("utf-8");
    #print(io)
    try:
        ioj=json.loads(io)
    except:
        ioj={"dt":"error"}
    #print(ioj)
    #print("------------")
    temp = ioj["dt"]
    if temp[0]!='e':
        if ioj["id"]=="430": # Z2 consumption
            sm2_new=ioj["c1"].rstrip("*kWh")
            if sm2!="0":
                sm2p = str(round((float(sm2_new)-float(sm2))*60000.0,3))
            sm2=sm2_new    
            sm2e=str(float(sm2)-float(sm2_mn))
            if sm2e[0]=='-':
                sm2e="0"
        if ioj["id"]=="568": # Z3 10kW
            sm3_new=ioj["p1"].rstrip("*kWh")
            if sm3!="0":
                sm3p = str(round((float(sm3_new)-float(sm3))*60000.0,3))
            sm3=sm3_new    
            sm3e=str(float(sm3)-float(sm3_mn))
            if sm3e[0]=='-':
                sm3e="0"
        if ioj["id"]=="423": # Z4 8kW
            sm4_new=ioj["p1"].rstrip("*kWh")
            if sm4!="0":
                sm4p = str(round((float(sm4_new)-float(sm4))*60000.0,3))
            sm4=sm4_new    
            sm4e=str(float(sm4)-float(sm4_mn))
            if sm4e[0]=='-':
                sm4e="0"
        if ioj["id"]=="552": # Z5 3kW
            sm5_new=ioj["p1"].rstrip("*kWh")
            if sm5!="0":
                sm5p = str(round((float(sm5_new)-float(sm5))*60000.0,3))
            sm5=sm5_new    
            sm5e=str(float(sm5)-float(sm5_mn))
            if sm5e[0]=='-':
                sm5e="0"
            sa_new=str(round(float(sm3)+float(sm4)+float(sm5),3))
            if sa!="0":
                sap = str(round((float(sa_new)-float(sa))*60000.0,3))
            sa=sa_new    
            sae=str(round(float(sm3e)+float(sm4e)+float(sm5e),3))
            try:    
                f = open (sm_p1_datafile1, 'r')
            except IOError:
                print("error reading file "+sm_p1_datafile1)
                return
            lineList = f.readlines()            #read all lines        
            f.close()
            try:    
                f = open (sm_p1_datafile1, 'a')
            except IOError:
                print ("Cannot create or find file: " + sm_p1_datafile1)
            try:    
                f2 = open (sm_p1_datafile2+ftime+'.min', 'a')
            except IOError:
                print ("Cannot create or find file: " + sm_p1_datafile2)
            if (len(lineList)) == 1:
                sm_p1_data = ' '.join((ftime3,sm2,sm3,sm4,sm5,sa,sm2e,sm3e,sm4e,sm5e,sae,sm2p,sm3p,sm4p,sm5p,sap))
                sm_p1_data = sm_p1_data + '\n'                                
            else:                        
                line = lineList[len(lineList)-1]    #get the last line
                lline =shlex.split(line)            #convert string (space seperated items) to list            
                sm_p1_data = ' '.join((ftime3,sm2,sm3,sm4,sm5,sa,sm2e,sm3e,sm4e,sm5e,sae,sm2p,sm3p,sm4p,sm5p,sap))
                sm_p1_data = sm_p1_data + '\n'
            print (sm_p1_data,end='')
            f.write(sm_p1_data)
            f2.write(sm_p1_data)
            f.close()
            f2.close()
    else:
        print("an error occured")

def sm_create_gp_file():
    """ The function prepares the gp file for plotting with gnuplot. First the
    old gp file is deleted. Then it uses the xx_gp_template.gp file in
    ~/../gp and replaces the keywords between the % sign by creating
    a new gp (xx.gp) file."""
    ftime2 = strftime("%d.%m.%y", localtime())
    Title = ftime2
    XFormat = '"%H:%M"'
    XTics = "60*60" #seconds
    Begin = ftime2 +" 00:00:01"
    End = ftime2 +" 23:59:59"
    Output = png_dir + "sm_p1_" + ftime + ".png"
    Input = sm_p1_datafile1
    try:
        os.remove(sm_p1_gnupfile2)
    except OSError:
        pass
    try:
        gf1 = open (sm_p1_gnupfile1,'r')
    except IOError:
        print ("Cannot find file: " + sm_p1_gnupfile1)
    try:
        gf2 = open (sm_p1_gnupfile2,'a')
    except IOError:
        print ("Cannot find file: " + sm_p1_gnupfile2)    
    gline1 = gf1.readline()
    while gline1 != "":
        if "%TITLE%" in gline1:
            gline1 = gline1.replace("%TITLE%",Title)
        if "%XFORMAT%" in gline1:
            gline1 = gline1.replace("%XFORMAT%",XFormat)
        if "%XTICS%" in gline1:
            gline1 = gline1.replace("%XTICS%",XTics)
        if "%BEGIN%" in gline1:
            gline1 = gline1.replace("%BEGIN%",Begin)
        if "%END%" in gline1:
            gline1 = gline1.replace("%END%",End)
        if "%OUTPUT%" in gline1:
            gline1 = gline1.replace("%OUTPUT%",Output)
        if "%INPUT%" in gline1:
            gline1 = gline1.replace("%INPUT%",Input)
        gf2.write(gline1)
        gline1 = gf1.readline()
    gf1.close()
    gf2.close()
    
    
    
def move_files(dir_src,dir_dst):
    for file in os.listdir(dir_src):
        if "sm_p1_" in file:
            src_file = os.path.join(dir_src, file)
            dst_file = os.path.join(dir_dst, file)
            shutil.move(src_file, dst_file)
   
#-----------------------------------------------------------------------------
# Main
#-----------------------------------------------------------------------------

try:    
    f = open (sm_p1_datafile1, 'r')
except IOError:
    try:    
        f = open (sm_p1_datafile1, 'w')
        sm_p1_data = ' '.join(("date", "time", "consumption [kWh]", "solar_10kW [kWh]",
                               "solar_8kW [kWh]", "solar_2.7kW [kWh]", "solar_all [kWh]",
                               "consumption day [Wh]", "solar_10kW day [Wh]",
                               "solar_8kW day [Wh]", "solar_2.7kW day [Wh]",
                               "solar_all day [Wh]", "consumption [W]", "solar_10kW [W]",
                               "solar_8kW [W]", "solar_2.7kW [W]", "solar_all [W]\n"))
        f.write(sm_p1_data)
        f.close()
    except IOError:
        print ("Cannot create or find file for smartmeter data")        
f.close()

mqttc = mqtt.Client(client_id=clientID, clean_session=True) # create client

mqttc.on_connect      = onConnect   # define the callback functions
mqttc.on_disconnect   = onDisconnect
mqttc.on_subscribe    = onSubscribe
mqttc.on_unsubscribe  = onUnsubscribe
mqttc.on_message      = onMessage

#mqttc.username_pw_set(config['MQTT']['userMQTT'], password=config['MQTT']['passwdMQTT'])
mqttc.connect(brokerIP, brokerPort, keepalive=60, bind_address="") # connect to the broker
mqttc.loop_start() # start loop to process callbacks! (new thread!) 

sm2 = sm3 = sm4 = sm5 = sa ="0"
sm2p = sm3p = sm4p = sm5p = sap = "0"
sm2e = sm3e = sm4e = sm5e = sae = "0"
sm2_mn = sm3_mn = sm4_mn = sm5_mn = "0"
flag_files=0
flag_new_header=0
flag_mail=0

try:
    while True:  
        now = datetime.now()
        now_time = now.time()
        rftime = (datetime.today() - timedelta(days=1)).strftime("%Y_%m_%d")
        sec=now.second
        if now_time >= time(23,59,58) and now_time < time(23,59,59): 
            flag_files=1
        if now_time >= time(0,0,0) and now_time < time(0,0,4): 
            flag_new_header=1
        if now_time >= time(1,10,10) and now_time < time(1,10,12): 
            flag_mail=1
        if sec==0:
            ftime4 = strftime("%H:%M", localtime())
            ftime = strftime("%Y_%m_%d", localtime())
            try:
                sm_create_gp_file()
            except:
                print ("cannot create sm_gp file")               
            try:
                p = subprocess.Popen(["/usr/bin/gnuplot", sm_p1_gnupfile2], stdin= subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                stdout, stderr = p.communicate('\r\n\r\n'.encode(), timeout=10)
                print ("gnuplot used for sm_p1  ",end='')
            except:
                print ("Error plotting file sm_p1 (gnuplot)")
            try:
                shutil.copy(png_dir+'sm_p1_'+ftime+'.png', wpath+'sm_p1_daily.png')
            except:
                print ("Cannot copy sm_p1 file")
            sleep(2.2)
        if flag_files==1: 
            print (now_time)
            try:
                os.remove(data_dir+'sm_p1_data.min')
            except OSError:
                print ("Error while trying to delete sm_p1_data.min")
            dir_src = data_dir
            dir_dst = data_a_dir
            move_files(dir_src,dir_dst)
            dir_src = png_dir
            dir_dst = png_a_dir
            move_files(dir_src,dir_dst)
            sleep(3)
            flag_files=0
        if flag_new_header==1: 
            print (now_time)
            try:    
                f = open (sm_p1_datafile1, 'r')
            except IOError:
                try:    
                    f = open (sm_p1_datafile1, 'w')
                    sm_p1_data = ' '.join(("date", "time", "consumption [kWh]", "solar_10kW [kWh]",
                                           "solar_8kW [kWh]", "solar_2.7kW [kWh]", "solar_all [kWh]",
                                           "consumption day [Wh]", "solar_10kW day [Wh]",
                                           "solar_8kW day [Wh]", "solar_2.7kW day [Wh]",
                                           "solar_all day [Wh]", "consumption [W]", "solar_10kW [W]",
                                           "solar_8kW [W]", "solar_2.7kW [W]", "solar_all [W]\n"))
                    f.write(sm_p1_data)
                    f.close()
                except IOError:
                    print ("Cannot create or find file for smartmeter data")        
            f.close()
            sleep(5)
            flag_new_header=0
        if flag_mail==1: 
            print (now_time)
            print(rftime)
            try:
                os.system('mpack -s "Mail from my beaglebone sm" -d '+ mailfile \
                          + ' ' + png_a_dir+'sm_p1_'+rftime+'.png' + ' my@my.com')
                print ("sending sm p1 mail")
            except:
                print ("Error mailing sm p1 png file")
            sleep(5)
            flag_mail=0
           
except KeyboardInterrupt:
    print("Keyboard interrupt by user")
    mqttc.loop_stop() # clean up
    mqttc.disconnect()

# End
